/*
 * This file is part of SubLink.
 * Copyright (C) 2009  Jeferson Hultmann <hultmann@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

var EXPORTED_SYMBOLS = ["buildCreateRedirMenu", "buildShortTabMenu"];

Components.utils.import("resource://sublink-migreme-modules/util.js");


function buildShortTabMenu(mainMenu, addr) {
  var doc = mainMenu.ownerDocument;
  var menuItem = doc.getElementById("sublink-migreme-convert");

  if (menuItem) {
    throw util.logStack("buildCreateRedirMenu " + menuItem);
  }


  var cmdData = {
    uri: util.stringToUri(addr),
  };

  var sepItem = mainMenu.insertBefore(doc.createElement("menuseparator"), mainMenu.firstChild);
  menuItem = mainMenu.insertBefore(doc.createElement("menuitem"), mainMenu.firstChild);
  menuItem.setAttribute("id", "sublink-migreme-convert");

  util.addClass(sepItem, "sublink-migreme-node");
  util.addClass(menuItem, "sublink-migreme-node");

  configLongUrlItem(cmdData, menuItem, "tab");
  var timeoutId = {};
  timeoutId.value = doc.defaultView.setTimeout(function() {
    configLongUrlFoundItems(mainMenu, cmdData, menuItem, "tab", timeoutId);
  }, 0);
  menuItem.setAttribute("sublink-migreme-timeoutID", timeoutId.value);
}


function buildCreateRedirMenu(mainMenu, cmdData) {
  var doc = mainMenu.ownerDocument;
  var menuItem = doc.getElementById("sublink-migreme-convert");

  if (menuItem) {
    throw util.logStack("buildCreateRedirMenu " + menuItem);
  }

  var sepItem = mainMenu.insertBefore(doc.createElement("menuseparator"), mainMenu.firstChild);
  menuItem = mainMenu.insertBefore(doc.createElement("menuitem"), mainMenu.firstChild);
  menuItem.setAttribute("id", "sublink-migreme-convert");

  util.addClass(sepItem, "sublink-migreme-node");
  util.addClass(menuItem, "sublink-migreme-node");

  var copy = cmdData.node.readOnly ? "copy" : "replace";
  configLongUrlItem(cmdData, menuItem, copy);
  var timeoutId = {};
  timeoutId.value = doc.defaultView.setTimeout(function() {
    configLongUrlFoundItems(mainMenu, cmdData, menuItem, copy, timeoutId);
  }, 0);
  menuItem.setAttribute("sublink-migreme-timeoutID", timeoutId.value);
}


//long==>short
function configLongUrlItem(cmdData, menuItem, action) {

  var menuUrl = formatUrl.crop(cmdData.uri);

  util.addClass(menuItem, "menuitem-iconic");
  menuItem.setAttribute("image", "http://migre.me/migreme.ico");

  switch (action) {
    case "tab":
      menuItem.setAttribute("label", util.getText("context-create.new.copyTab.label", "migre.me"));
      menuItem.setAttribute("accesskey", util.getText("context-create.new.copyTab.accesskey"));
      break;
    case "copy":
      menuItem.setAttribute("label", util.getText("context-create.new.copy.label", "migre.me"));
      menuItem.setAttribute("accesskey", util.getText("context-create.new.copy.accesskey"));
      menuItem.setAttribute("description", menuUrl.item);
      break;
    case "replace":
      menuItem.setAttribute("label", util.getText("context-create.new.replace.label", "migre.me"));
      menuItem.setAttribute("accesskey", util.getText("context-create.new.replace.accesskey"));
      menuItem.setAttribute("description", menuUrl.item);
      break;
    default:
      throw "configLongUrlItem"
  }


  if (util.isOffline()) {
    menuItem.setAttribute("disabled", "true");
    return;
  }


  var onCommand;
  var browser = menuItem.ownerDocument.defaultView.getBrowser().selectedBrowser;


  switch (action) {
    case "tab":
    case "copy":
      // BUG nao mostra cursor busy
      onCommand = function(evt) {
        function onSuccess(info) {
          util.toClipboard(info.shortUri.spec);
        }

        util.toClipboard("");
        util.toTinyUrlRequest(cmdData.uri, browser, onSuccess);
      };
      break;
    case "replace":
      onCommand = function(evt) {
        function onSuccess(info) {
          Components.utils.import("resource://sublink-migreme-modules/run-input.js");
          PUBLIC_replaceCore(cmdData, info.shortUri.spec, browser);
        }

        util.toTinyUrlRequest(cmdData.uri, browser, onSuccess);
      };
      break;
    default:
      throw "configLongUrlItem 2"
  }


  menuItem.addEventListener("command", onCommand, false);


  /*
  var title = util.pageTitle(cmdData.uri);
  var tooltip;
  if (title) {
    tooltip = title + "\n" + menuUrl.tooltip;
  } else {
    tooltip = menuUrl.tooltip;
  }
  */
  menuItem.setAttribute("tooltiptext", menuUrl.tooltip);
}


function configLongUrlFoundItems(mainMenu, cmdData, menuItem, action, timeoutId) {
  var doc = mainMenu.ownerDocument;
  var menuItem = doc.getElementById("sublink-migreme-convert");
  if (menuItem === null) {
    // menu is gone
    return;
  }

  if (menuItem.getAttribute("sublink-migreme-timeoutID") !== timeoutId.value.toString()) {
    return;
  }


  Components.utils.import("resource://sublink-migreme-modules/redir-storage.js");
  var shortUrls = redirStorage.loadShortUrls(cmdData.uri);

  for (var shortUrl in shortUrls) {
    if (shortUrls.hasOwnProperty(shortUrl) === false) {
      util.log("configLongUrlFoundItems " + shortUrl);
      return;
    }
    var props = shortUrls[shortUrl];
    util.log("props="+props.toSource());


    var node = mainMenu.insertBefore(doc.createElement("menuitem"), menuItem);
    util.addClass(node, "sublink-migreme-node");
    var handler = new SavedRedirCommand(action, cmdData, shortUrl);
    node.addEventListener("command", handler, false);
    node.addEventListener("click", handler, false);


    switch (action) {
      case "tab":
        node.setAttribute("label", util.getText("context-create.saved.copyTab.label", shortUrl));
        node.setAttribute("accesskey", util.getText("context-create.saved.copyTab.accesskey"));
        break;
      case "copy":
        node.setAttribute("label", util.getText("context-create.saved.copy.label", shortUrl));
        node.setAttribute("accesskey", util.getText("context-create.saved.copy.accesskey"));
        break;
      case "replace":
        node.setAttribute("label", util.getText("context-create.saved.replace.label", shortUrl));
        node.setAttribute("accesskey", util.getText("context-create.saved.replace.accesskey"));
        break;
      default:
        throw "configLongUrlItem 3"
    }


  }

}



function SavedRedirCommand(action, cmdData, shortUrl) {
  this._cmdData = cmdData;
  this._action = action;
  this._url = shortUrl;
}

SavedRedirCommand.prototype = {
  handleEvent: function(evt) {
    var menuItem = evt.target;
    util.log("SavedRedirCommand: "+menuItem.tagName+evt.originalTarget.tagName);

    switch (evt.type) {
      case "click":
        var infoUrl = util.getInfoUrl(util.stringToUri(this._url));
        if (infoUrl) {
          util.openPage2(evt, infoUrl, true);
        }
        return;

      case "command":
        switch (this._action) {
          case "copy":
          case "tab":
            util.toClipboard(this._url);
            break;
          case "replace":
            var browser = menuItem.ownerDocument.defaultView.getBrowser().selectedBrowser;
            Components.utils.import("resource://sublink-migreme-modules/run-input.js");
            PUBLIC_replaceCore(this._cmdData, this._url, browser);
            break;
          default:
            throw "SavedRedirCommand";
        }
        return;

      default:
        throw util.logStack(evt.type);
    }
  }
};


var formatUrl = {
  crop: function(uri) {
    var max = 30;
    var full = uri.spec;
    if (full.length < max || util.isRedirUri(uri)) {
      return {item: full, tooltip: ""};
    }

    // without http://
    var txt = full.replace(/^https?:\/\//i, "");
    if (uri.path.length === 1) {
      txt = txt.substr(0, txt.length - 1); // remove trailing '/'
    }
    if (txt.length < max) {
      return {item:txt, tooltip:full};
    }

    // no path: send domain
    if (uri.path.length === 1) { // = '/'
      return {item:txt, tooltip:full};
    }

    /*
    // without www.
    txt = txt.replace(/^www\./i, '');
    if (txt.length < max) {
      return {item:txt, tooltip:full};
    }*/

    //var maxPath = 20;
    //max -= uri.host.length;

    // z.com/xx-ddd-a
    var len = txt.indexOf("/");

    // small path
    if ((len + uri.path.length) <= max) {
      return {item:txt, tooltip:full};
    }

    // 012345678901234567890123456789
    // br.mozdev.org                 len    = 13
    // /firefox/backup-dos-favoritos lenPath= 29

    // trim at words
    var lenPath = 0;
    var lenTrim;
    do {
      lenTrim = lenPath;
      lenPath = this._indexOfNextStopWord(uri.path, lenPath + 1);
      if (lenPath === -1) {
        break;
      }
    } while((len + lenPath) <= max);


    // trim path
    var uri2 = uri.clone();
    uri2.path = uri2.path.substr(0, lenTrim);
    txt = uri2.spec.replace(/^https?:\/\//i, '');
    //txt = txt.replace(/^www\./i, '');
    return {item:txt + "\u2026", tooltip:full};
  },

  _indexOfNextStopWord: function(path, idxStart) {
    var idx = this._indexOf(path, ["/", "&", ".", "?", "-", "_"], idxStart);
    return idx === Infinity ? -1 : idx;

    /*
    if (idx !== Infinity) {
      return idx;
    }
    */
    /*
    var idx = this._indexOf(path, ['/', '&'], idxStart);
    if (idx !== Infinity) {
      return idx;
    }

    idx = this._indexOf(path, ['.', '?'], idxStart);
    if (idx !== Infinity) {
      return idx;
    }*/

    //return -1;
  },

  _indexOf: function(path, bounds, idxStart) {
    var mySort = function(a, b) {
      return a - b;
    };
    var stopChars = bounds.map(function(myChar) {
      var idx2 = path.indexOf(myChar, idxStart);
      return idx2 === -1 ? Infinity : idx2;
    });
    stopChars.sort(mySort);
    return stopChars[0];
  }

};
